#define com_maroontress_libcgi_util_Tub_IMPORT
#define com_maroontress_libcgi_db_XML_IMPORT
#define com_maroontress_libcgi_db_Box_IMPORT

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <com/maroontress/libcgi/db/XML.h>
#include <com/maroontress/libcgi/db/Box.h>
#include <com/maroontress/libcgi/util/Tub.h>
#include "Environment.h"

struct Environment {
    struct Tub *tub;
    unsigned int size;
};

void
Environment_free(struct Environment *e)
{
    if (e == NULL)
	return;
    Tub_free(e->tub);
    free(e);
}

struct Environment *
Environment_create(void)
{
    struct Environment *e;

    if ((e = (struct Environment *)malloc(sizeof(*e))) == NULL)
	return NULL;
    e->size = 0;
    e->tub = NULL;

    if ((e->tub = Tub_create()) == NULL) {
	Environment_free(e);
	return NULL;
    }
    return e;
}

int
Environment_setXML(struct Environment *e, const char *name, const char *value)
{
    size_t saved;

    if (value == NULL)
	return 0;
    saved = Tub_getSize(e->tub);
    if (Tub_writeCString(e->tub, name) < 0
	|| Tub_writeByte(e->tub, '=') < 0
	|| XML_writeEscapedToTub(e->tub, value) < 0
	|| Tub_writeByte(e->tub, 0) < 0) {
	Tub_setSize(e->tub, saved);
	return -1;
    }
    ++(e->size);
    return 0;
}

int
Environment_setRaw(struct Environment *e, const char *name, const char *value)
{
    size_t saved;

    if (value == NULL)
	return 0;
    saved = Tub_getSize(e->tub);
    if (Tub_writeCString(e->tub, name) < 0
	|| Tub_writeByte(e->tub, '=') < 0
	|| Tub_writeCString(e->tub, value) < 0
	|| Tub_writeByte(e->tub, 0) < 0) {
	Tub_setSize(e->tub, saved);
	return -1;
    }
    ++(e->size);
    return 0;
}

static int
setAnyByBox(struct Environment *e, const char *envName,
	    struct Box *box, const char *boxName,
	    int (*writer)(struct Box *, const char *, struct Tub *))
{
    size_t saved;

    saved = Tub_getSize(e->tub);
    if (Tub_writeCString(e->tub, envName) < 0
	|| Tub_writeByte(e->tub, '=') < 0
	|| writer(box, boxName, e->tub) < 0
	|| Tub_writeByte(e->tub, 0) < 0) {
	Tub_setSize(e->tub, saved);
	return -1;
    }
    ++(e->size);
    return 0;
}

int
Environment_setXMLByBox(struct Environment *e, const char *envName,
			struct Box *box, const char *boxName)
{
    return setAnyByBox(e, envName, box, boxName, Box_writeXMLEscapedToTub);
}

int
Environment_setRawByBox(struct Environment *e, const char *envName,
			struct Box *box, const char *boxName)
{
    return setAnyByBox(e, envName, box, boxName, Box_writeToTub);
}

int
Environment_copyXMLByBox(struct Environment *e, const char *envName,
			 struct Box *box)
{
    return Environment_setXMLByBox(e, envName, box, envName);
}

struct Argument {
    char **env;
    char *data;
};

static void
Argument_initialize(struct Argument *a)
{
    a->env = NULL;
    a->data = NULL;
}

static void
Argument_finalize(struct Argument *a)
{
    free(a->env);
    free(a->data);
}

int
Environment_execute(struct Environment *e, const char *const *arg)
{
    struct Argument a;
    unsigned int k;
    char *data;

    Argument_initialize(&a);
    if ((a.env = malloc(sizeof(char *) * (e->size + 1))) == NULL
	|| (a.data = Tub_createByteArray(e->tub)) == NULL) {
	Argument_finalize(&a);
	return -1;
    }
    data = a.data;
    for (k = 0; k < e->size; ++k) {
	a.env[k] = data;
	data += strlen(data) + 1;
    }
    a.env[k] = NULL;
    (void)execve(arg[0], (char *const *)arg, a.env);
    Argument_finalize(&a);
    return -1;
}
