#define com_maroontress_libcgi_db_Tree_IMPORT
#define com_maroontress_libcgi_util_Tub_IMPORT

#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <com/maroontress/libcgi/db/Tree.h>
#include <com/maroontress/libcgi/util/Tub.h>
#include "Project.h"

#define BUILD_DB "build.db"

struct Project {
    char *baseDir;
    char dbPath[MAXPATHLEN];
};

static int
writeChar(struct Tub *t, int c)
{
    char buf[4];

    if (isalnum(c)) {
	return Tub_writeByte(t, c);
    }
    snprintf(buf, sizeof(buf), "%%%.2x", c);
    return Tub_writeCString(t, buf);
}

static int
writeTranslatedID(struct Tub *t, const unsigned char *id)
{
    if (Tub_writeCString(t, "projects/") < 0)
	return -1;
    for (; *id != 0 && writeChar(t, *id) == 0; ++id)
	;
    return ((*id != 0) ? -1 : Tub_writeByte(t, 0));
}

static char *
createBasePath(const char *id)
{
    struct Tub *t;
    char *path;

    if ((t = Tub_create()) == NULL)
	return NULL;
    path = ((writeTranslatedID(t, id) < 0) ? NULL : Tub_createByteArray(t));
    Tub_free(t);
    return path;
}

struct Project *
Project_create(const char *projectID)
{
    struct Project *pj;

    if ((pj = (struct Project *)malloc(sizeof(*pj))) == NULL) {
	return NULL;
    }
    pj->baseDir = NULL;

    if ((pj->baseDir = createBasePath(projectID)) == NULL) {
	Project_free(pj);
	return NULL;
    }
    snprintf(pj->dbPath, sizeof(pj->dbPath), "%s/%s", pj->baseDir, BUILD_DB);
    return pj;
}

void
Project_free(struct Project *pj)
{
    if (pj == NULL)
	return;
    free(pj->baseDir);
    free(pj);
}

char *
Project_getBasePath(struct Project *pj)
{
    return pj->baseDir;
}

char *
Project_getDBPath(struct Project *pj)
{
    return pj->dbPath;
}

struct Tree *
Project_createTree(struct Project *pj, const struct Tree_Mode *mode)
{
    return Tree_create(Project_getDBPath(pj), mode);
}

int
Project_removeDir(struct Project *pj)
{
    /* Ȥꤢ */
    char cmd[MAXPATHLEN];

    snprintf(cmd, sizeof(cmd), "rm -rf %s", Project_getBasePath(pj));
    system(cmd);
    return 0;
}

int
Project_makeDir(struct Project *pj)
{
    return mkdir(Project_getBasePath(pj), 0755);
}
